@@ -8,10 +8,21 @@ from copy import deepcopy
 from datetime import datetime
 from io import BytesIO
 from pathlib import Path
-from typing import TYPE_CHECKING, Any, BinaryIO, Iterator, Literal, Optional, Union
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    BinaryIO,
+    Iterator,
+    Literal,
+    Optional,
+    Sequence,
+    TypedDict,
+    Union,
+)
 from urllib.parse import quote
 
 import httpx
+from typing_extensions import Required, TypeGuard
 
 if TYPE_CHECKING:
     import pandas as pd
@@ -23,26 +34,40 @@ logger = logging.getLogger(__name__)
 
 DatasetExample = v1.DatasetExample
 
+
+class _InputDatasetExample(TypedDict, total=False):
+    """
+    This type is created manually since we do not have compiled request types
+    for the POST /v1/datasets/upload route.
+
+    https://github.com/Arize-ai/phoenix/blob/19e69091543b9c0f4051b9e561fa53d4f39d0fa4/src/phoenix/server/api/routers/v1/datasets.py#L354
+    """
+
+    input: Required[Mapping[str, Any]]
+    output: Required[Mapping[str, Any]]
+    metadata: Mapping[str, Any]
+
+
 DEFAULT_TIMEOUT_IN_SECONDS = 5
 
 
-def _is_valid_dataset_example(obj: Any) -> bool:
-    """Check if an object is a valid DatasetExample using the TypedDict's annotations.
-
-    Args:
-        obj (Any): The object to validate.
-
-    Returns:
-        bool: True if the object is a valid DatasetExample, False otherwise.
+def _is_input_dataset_example(obj: Any) -> TypeGuard[_InputDatasetExample]:
+    """
+    Checks if an object is a valid _InputDatasetExample.
     """
     if not isinstance(obj, dict):
         return False
 
-    required_fields = set(DatasetExample.__annotations__.keys())
+    keys = set(obj.keys())  # pyright: ignore[reportUnknownArgumentType, reportUnknownVariableType]
+    required_keys = {"input", "output"}
+    return required_keys.issubset(keys)  # pyright: ignore[reportUnknownArgumentType]
 
-    if not required_fields.issubset(obj.keys()):  # pyright: ignore[reportUnknownArgumentType]
-        return False
-    return True
+
+def _is_iterable_of_input_dataset_examples(obj: Any) -> TypeGuard[Iterable[_InputDatasetExample]]:
+    """
+    Checks if an object is an iterable of _InputDatasetExample objects.
+    """
+    return isinstance(obj, Iterable) and all(_is_input_dataset_example(example) for example in obj)  # pyright: ignore[reportUnknownVariableType]
 
 
 class Dataset:
@@ -58,6 +83,7 @@ class Dataset:
         created_at (datetime): When the dataset was created.
         updated_at (datetime): When the dataset was last updated.
         example_count (int): Number of examples in this version.
+        split_names (list[str]): List of dataset split names.
     """
 
     def __init__(
@@ -88,6 +114,11 @@ class Dataset:
         """The current version ID."""
         return self._examples_data["version_id"]
 
+    @property
+    def _filtered_split_names(self) -> list[str]:
+        """The dataset split names."""
+        return list(self._examples_data.get("filtered_splits", []))
+
     @property
     def examples(self) -> list[DatasetExample]:
         """List of examples in this version."""
@@ -115,8 +146,6 @@ class Dataset:
     @property
     def example_count(self) -> int:
         """Number of examples in this version."""
-        if "example_count" in self._dataset_info:
-            return self._dataset_info["example_count"]
         return len(self.examples)
 
     def __repr__(self) -> str:
@@ -195,7 +224,7 @@ class Dataset:
             json_data = dataset.to_dict()
             restored = Dataset.from_dict(json_data)
         """
-        return {
+        result: dict[str, Any] = {
             "id": self.id,
             "name": self.name,
             "description": self.description,
@@ -208,6 +237,12 @@ class Dataset:
             "examples": deepcopy(self.examples),
         }
 
+        # Include filtered_splits if present (optional field for backwards compatibility)
+        if self._filtered_split_names:
+            result["filtered_splits"] = self._filtered_split_names
+
+        return result
+
     @classmethod
     def from_dict(cls, json_data: dict[str, Any]) -> "Dataset":
         """
@@ -254,11 +289,15 @@ class Dataset:
         if json_data.get("example_count") is not None:
             dataset_info["example_count"] = json_data["example_count"]
 
-        examples_data = {
+        examples_data: dict[str, Any] = {
             "version_id": json_data["version_id"],
             "examples": deepcopy(json_data["examples"]),
         }
 
+        # Handle optional split_ids for backwards compatibility
+        if "filtered_splits" in json_data:
+            examples_data["filtered_splits"] = json_data["filtered_splits"]
+
         return cls(dataset_info, examples_data)  # type: ignore[arg-type]
 
 
@@ -276,11 +315,16 @@ class DatasetKeys:
     """
 
     def __init__(
-        self, input_keys: frozenset[str], output_keys: frozenset[str], metadata_keys: frozenset[str]
+        self,
+        input_keys: frozenset[str],
+        output_keys: frozenset[str],
+        metadata_keys: frozenset[str],
+        split_keys: frozenset[str] = frozenset(),
     ):
         self.input = input_keys
         self.output = output_keys
         self.metadata = metadata_keys
+        self.split = split_keys
 
         if self.input & self.output:
             raise ValueError(f"Input and output keys overlap: {self.input & self.output}")
@@ -288,16 +332,22 @@ class DatasetKeys:
             raise ValueError(f"Input and metadata keys overlap: {self.input & self.metadata}")
         if self.output & self.metadata:
             raise ValueError(f"Output and metadata keys overlap: {self.output & self.metadata}")
+        if self.input & self.split:
+            raise ValueError(f"Input and split keys overlap: {self.input & self.split}")
+        if self.output & self.split:
+            raise ValueError(f"Output and split keys overlap: {self.output & self.split}")
+        if self.metadata & self.split:
+            raise ValueError(f"Metadata and split keys overlap: {self.metadata & self.split}")
 
     def check_differences(self, available_keys: frozenset[str]) -> None:
         """Check that all specified keys exist in available keys."""
-        all_keys = self.input | self.output | self.metadata
+        all_keys = self.input | self.output | self.metadata | self.split
         if diff := all_keys - available_keys:
             raise ValueError(f"Keys not found in available columns: {diff}")
 
     def __iter__(self) -> "Iterator[str]":
         """Allow iteration over all keys."""
-        return iter(self.input | self.output | self.metadata)
+        return iter(self.input | self.output | self.metadata | self.split)
 
 
 def _parse_datetime(datetime_str: str) -> datetime:
@@ -414,6 +464,7 @@ class Datasets:
         *,
         dataset: DatasetIdentifier,
         version_id: Optional[str] = None,
+        splits: Optional[Sequence[str]] = None,
         timeout: Optional[int] = DEFAULT_TIMEOUT_IN_SECONDS,
     ) -> Dataset:
         """
@@ -425,33 +476,40 @@ class Datasets:
 
         Args:
             dataset (DatasetIdentifier): Dataset identifier - can be a dataset
-            ID string, name string,
-                Dataset object, or dict with 'id'/'name' fields.
+                ID string, name string, Dataset object, or dict with 'id'/'name' fields.
             version_id (Optional[str]): Specific version ID of the dataset. If
-            None, returns the
-                latest version.
+                None, returns the latest version.
+            splits (Optional[list[str]]): List of dataset split names to filter by.
+                If provided, only returns examples that belong to the specified splits.
             timeout (Optional[int]): Request timeout in seconds (default: 5).
 
         Returns:
             Dataset: Dataset object containing complete dataset metadata and all
-            examples. The dataset can be iterated over, converted to DataFrame,
-            or accessed by index.
+                examples. The dataset can be iterated over, converted to DataFrame,
+                or accessed by index.
 
         Raises:
-            ValueError: If dataset identifier format is invalid or dataset not
-            found. httpx.HTTPStatusError: If the API request fails.
+            ValueError: If dataset identifier format is invalid or dataset not found.
+            httpx.HTTPStatusError: If the API request fails.
 
         Example::
 
-            from phoenix.client import Client client = Client()
+            from phoenix.client import Client
+            client = Client()
 
-            # Get dataset by name dataset =
-            client.datasets.get_dataset(dataset="my-dataset") print(f"Dataset
-            {dataset.name} has {len(dataset)} examples")
+            # Get dataset by name
+            dataset = client.datasets.get_dataset(dataset="my-dataset")
+            print(f"Dataset {dataset.name} has {len(dataset)} examples")
 
-            # Get specific version versioned = client.datasets.get_dataset(
+            # Get specific version
+            versioned = client.datasets.get_dataset(
                 dataset="my-dataset", version_id="version-123"
             )
+
+            # Get dataset filtered by splits
+            train_data = client.datasets.get_dataset(
+                dataset="my-dataset", splits=["train", "validation"]
+            )
         """
         resolved_id, resolved_name = self._resolve_dataset_id_and_name(dataset, timeout=timeout)
 
@@ -471,9 +529,13 @@ class Datasets:
         dataset_response.raise_for_status()
         dataset_info = dataset_response.json()["data"]
 
-        params = {"version_id": version_id} if version_id else None
+        params: dict[str, Union[str, Sequence[str]]] = (
+            {"version_id": version_id} if version_id else {}
+        )
+        if splits and len(splits) > 0:
+            params["split"] = splits
         examples_response = self._client.get(
-            url=f"v1/datasets/{quote(dataset_id)}/examples",
+            url=f"v1/datasets/{quote(dataset_id)}/examples",  # ?version_id=...&split=...&split=...
             params=params,
             headers={"accept": "application/json"},
             timeout=timeout,
@@ -664,12 +726,13 @@ class Datasets:
         self,
         *,
         name: str,
-        examples: Optional[Union[DatasetExample, Iterable[DatasetExample]]] = None,
+        examples: Optional[Union[Mapping[str, Any], Iterable[Mapping[str, Any]]]] = None,
         dataframe: Optional["pd.DataFrame"] = None,
         csv_file_path: Optional[Union[str, Path]] = None,
         input_keys: Iterable[str] = (),
         output_keys: Iterable[str] = (),
         metadata_keys: Iterable[str] = (),
+        split_keys: Iterable[str] = (),
         inputs: Iterable[Mapping[str, Any]] = (),
         outputs: Iterable[Mapping[str, Any]] = (),
         metadata: Iterable[Mapping[str, Any]] = (),
@@ -681,13 +744,15 @@ class Datasets:
 
         Args:
             dataset_name: Name of the dataset.
-            examples: Either a single DatasetExample or list of DatasetExample objects to add.
+            examples: Either a single dictionary with required 'input' and 'output' keys
+                and an optional 'metadata' key, or an iterable of such dictionaries.
                 When provided, inputs/outputs/metadata are extracted automatically.
             dataframe: pandas DataFrame (requires pandas to be installed).
             csv_file_path: Location of a CSV text file
             input_keys: List of column names used as input keys.
             output_keys: List of column names used as output keys.
             metadata_keys: List of column names used as metadata keys.
+            split_keys: List of column names used for automatically assigning examples to splits.
             inputs: List of dictionaries each corresponding to an example.
             outputs: List of dictionaries each corresponding to an example.
             metadata: List of dictionaries each corresponding to an example.
@@ -715,16 +780,23 @@ class Datasets:
         if dataframe is not None and csv_file_path is not None:
             raise ValueError("Please provide either dataframe or csv_file_path, but not both")
 
+        splits_from_examples: list[Any] = []
         if examples is not None:
-            examples_list: list[DatasetExample]
-            if _is_valid_dataset_example(examples):
-                examples_list = [examples]  # type: ignore[list-item]
+            examples_list: list[_InputDatasetExample]
+            if _is_input_dataset_example(examples):
+                examples_list = [examples]
+            elif _is_iterable_of_input_dataset_examples(examples):
+                examples_list = list(examples)
             else:
-                examples_list = list(examples)  # type: ignore[arg-type]
+                raise ValueError(
+                    "examples must be a single dictionary with required 'input' and 'output' keys "
+                    "and an optional 'metadata' key, or an iterable of such dictionaries"
+                )
 
             inputs = [dict(example["input"]) for example in examples_list]
             outputs = [dict(example["output"]) for example in examples_list]
-            metadata = [dict(example["metadata"]) for example in examples_list]
+            metadata = [dict(example.get("metadata", {})) for example in examples_list]
+            splits_from_examples = [example.get("splits", None) for example in examples_list]
 
         if has_tabular:
             table = dataframe if dataframe is not None else csv_file_path
@@ -735,6 +807,7 @@ class Datasets:
                 input_keys=input_keys,
                 output_keys=output_keys,
                 metadata_keys=metadata_keys,
+                split_keys=split_keys,
                 dataset_description=dataset_description,
                 action="create",
                 timeout=timeout,
@@ -745,6 +818,7 @@ class Datasets:
                 inputs=inputs,
                 outputs=outputs,
                 metadata=metadata,
+                splits=splits_from_examples if examples is not None else [],
                 dataset_description=dataset_description,
                 action="create",
                 timeout=timeout,
@@ -754,12 +828,13 @@ class Datasets:
         self,
         *,
         dataset: DatasetIdentifier,
-        examples: Optional[Union[DatasetExample, Iterable[DatasetExample]]] = None,
+        examples: Optional[Union[Mapping[str, Any], Iterable[Mapping[str, Any]]]] = None,
         dataframe: Optional["pd.DataFrame"] = None,
         csv_file_path: Optional[Union[str, Path]] = None,
         input_keys: Iterable[str] = (),
         output_keys: Iterable[str] = (),
         metadata_keys: Iterable[str] = (),
+        split_keys: Iterable[str] = (),
         inputs: Iterable[Mapping[str, Any]] = (),
         outputs: Iterable[Mapping[str, Any]] = (),
         metadata: Iterable[Mapping[str, Any]] = (),
@@ -771,13 +846,15 @@ class Datasets:
         Args:
             dataset: A dataset identifier - can be a dataset ID string, name string,
                 Dataset object, or dict with 'id'/'name' fields.
-            examples: Either a single DatasetExample or list of DatasetExample objects to add.
+            examples: Either a single dictionary with required 'input' and 'output' keys
+                and an optional 'metadata' key, or an iterable of such dictionaries.
                 When provided, inputs/outputs/metadata are extracted automatically.
             dataframe: pandas DataFrame (requires pandas to be installed).
             csv_file_path: Location of a CSV text file
             input_keys: List of column names used as input keys.
             output_keys: List of column names used as output keys.
             metadata_keys: List of column names used as metadata keys.
+            split_keys: List of column names used for automatically assigning examples to splits.
             inputs: List of dictionaries each corresponding to an example.
             outputs: List of dictionaries each corresponding to an example.
             metadata: List of dictionaries each corresponding to an example.
@@ -821,16 +898,23 @@ class Datasets:
         if dataframe is not None and csv_file_path is not None:
             raise ValueError("Please provide either dataframe or csv_file_path, but not both")
 
+        splits_from_examples: list[Any] = []
         if examples is not None:
-            examples_list: list[DatasetExample]
-            if _is_valid_dataset_example(examples):
-                examples_list = [examples]  # type: ignore[list-item]
+            examples_list: list[_InputDatasetExample]
+            if _is_input_dataset_example(examples):
+                examples_list = [examples]
+            elif _is_iterable_of_input_dataset_examples(examples):
+                examples_list = list(examples)
             else:
-                examples_list = list(examples)  # type: ignore[arg-type]
+                raise ValueError(
+                    "examples must be a single dictionary with required 'input' and 'output' keys "
+                    "and an optional 'metadata' key, or an iterable of such dictionaries"
+                )
 
             inputs = [dict(example["input"]) for example in examples_list]
             outputs = [dict(example["output"]) for example in examples_list]
-            metadata = [dict(example["metadata"]) for example in examples_list]
+            metadata = [dict(example.get("metadata", {})) for example in examples_list]
+            splits_from_examples = [example.get("splits") for example in examples_list]
 
         if has_tabular:
             table = dataframe if dataframe is not None else csv_file_path
@@ -841,6 +925,7 @@ class Datasets:
                 input_keys=input_keys,
                 output_keys=output_keys,
                 metadata_keys=metadata_keys,
+                split_keys=split_keys,
                 dataset_description=None,
                 action="append",
                 timeout=timeout,
@@ -851,6 +936,7 @@ class Datasets:
                 inputs=inputs,
                 outputs=outputs,
                 metadata=metadata,
+                splits=splits_from_examples if examples is not None else [],
                 dataset_description=None,
                 action="append",
                 timeout=timeout,
@@ -900,6 +986,7 @@ class Datasets:
         input_keys: Iterable[str],
         output_keys: Iterable[str] = (),
         metadata_keys: Iterable[str] = (),
+        split_keys: Iterable[str] = (),
         dataset_description: Optional[str] = None,
         action: Literal["create", "append"] = "create",
         timeout: Optional[int] = DEFAULT_TIMEOUT_IN_SECONDS,
@@ -910,6 +997,7 @@ class Datasets:
         input_keys_set = frozenset(input_keys)
         output_keys_set = frozenset(output_keys)
         metadata_keys_set = frozenset(metadata_keys)
+        split_keys_set = frozenset(split_keys)
 
         # Auto-infer keys if none provided
         if not any([input_keys_set, output_keys_set, metadata_keys_set]):
@@ -918,7 +1006,7 @@ class Datasets:
             output_keys_set = frozenset(output_keys_tuple)
             metadata_keys_set = frozenset(metadata_keys_tuple)
 
-        keys = DatasetKeys(input_keys_set, output_keys_set, metadata_keys_set)
+        keys = DatasetKeys(input_keys_set, output_keys_set, metadata_keys_set, split_keys_set)
 
         if isinstance(table, Path) or isinstance(table, str):
             file = _prepare_csv(Path(table), keys)
@@ -945,6 +1033,7 @@ class Datasets:
                 "input_keys[]": sorted(keys.input),
                 "output_keys[]": sorted(keys.output),
                 "metadata_keys[]": sorted(keys.metadata),
+                "split_keys[]": sorted(keys.split),
             },
             params={"sync": True},
             headers={"accept": "application/json"},
@@ -960,6 +1049,7 @@ class Datasets:
         inputs: Iterable[Mapping[str, Any]],
         outputs: Iterable[Mapping[str, Any]] = (),
         metadata: Iterable[Mapping[str, Any]] = (),
+        splits: Iterable[Any] = (),
         dataset_description: Optional[str] = None,
         action: Literal["create", "append"] = "create",
         timeout: Optional[int] = DEFAULT_TIMEOUT_IN_SECONDS,
@@ -971,6 +1061,7 @@ class Datasets:
         inputs_list = list(inputs)
         outputs_list = list(outputs) if outputs else []
         metadata_list = list(metadata) if metadata else []
+        splits_list = list(splits) if splits else []
 
         if not inputs_list:
             raise ValueError("inputs must be non-empty")
@@ -978,7 +1069,10 @@ class Datasets:
         if not _is_all_dict(inputs_list):
             raise ValueError("inputs must contain only dictionaries")
 
-        for name, data in [("outputs", outputs_list), ("metadata", metadata_list)]:
+        for name, data in [
+            ("outputs", outputs_list),
+            ("metadata", metadata_list),
+        ]:
             if data:
                 if len(data) != len(inputs_list):
                     raise ValueError(
@@ -988,12 +1082,19 @@ class Datasets:
                 if not _is_all_dict(data):
                     raise ValueError(f"{name} must contain only dictionaries")
 
+        # Validate splits separately (can be string, list of strings, or None)
+        if splits_list and len(splits_list) != len(inputs_list):
+            raise ValueError(
+                f"splits must have same length as inputs ({len(splits_list)} != {len(inputs_list)})"
+            )
+
         payload = {
             "action": action,
             "name": dataset_name,
             "inputs": inputs_list,
             "outputs": outputs_list or [{}] * len(inputs_list),
             "metadata": metadata_list or [{}] * len(inputs_list),
+            "splits": splits_list or [None] * len(inputs_list),
         }
 
         if dataset_description is not None:
@@ -1131,6 +1232,7 @@ class AsyncDatasets:
         *,
         dataset: DatasetIdentifier,
         version_id: Optional[str] = None,
+        splits: Optional[Sequence[str]] = None,
         timeout: Optional[int] = DEFAULT_TIMEOUT_IN_SECONDS,
     ) -> Dataset:
         """
@@ -1141,6 +1243,8 @@ class AsyncDatasets:
             dataset: A dataset identifier - can be a dataset ID string, name string,
                 Dataset object, or dict with 'id'/'name' fields.
             version_id: An ID for the version of the dataset, or None.
+            splits: Optional list of dataset split names to filter by.
+                If provided, only returns examples that belong to the specified splits.
             timeout: Optional request timeout in seconds.
 
         Returns:
@@ -1170,7 +1274,11 @@ class AsyncDatasets:
         dataset_response.raise_for_status()
         dataset_info = dataset_response.json()["data"]
 
-        params = {"version_id": version_id} if version_id else None
+        params: dict[str, Union[str, Sequence[str]]] = (
+            {"version_id": version_id} if version_id else {}
+        )
+        if splits and len(splits) > 0:
+            params["split"] = splits
         examples_response = await self._client.get(
             url=f"v1/datasets/{quote(dataset_id)}/examples",
             params=params,
@@ -1361,12 +1469,13 @@ class AsyncDatasets:
         self,
         *,
         name: str,
-        examples: Optional[Union[DatasetExample, Iterable[DatasetExample]]] = None,
+        examples: Optional[Union[Mapping[str, Any], Iterable[Mapping[str, Any]]]] = None,
         dataframe: Optional["pd.DataFrame"] = None,
         csv_file_path: Optional[Union[str, Path]] = None,
         input_keys: Iterable[str] = (),
         output_keys: Iterable[str] = (),
         metadata_keys: Iterable[str] = (),
+        split_keys: Iterable[str] = (),
         inputs: Iterable[Mapping[str, Any]] = (),
         outputs: Iterable[Mapping[str, Any]] = (),
         metadata: Iterable[Mapping[str, Any]] = (),
@@ -1378,13 +1487,15 @@ class AsyncDatasets:
 
         Args:
             dataset_name: Name of the dataset.
-            examples: Either a single DatasetExample or list of DatasetExample objects to add.
-                When provided, inputs/outputs/metadata are extracted automatically.
+            examples: Either a single dictionary with required 'input' and 'output' keys
+                and an optional 'metadata' key, or an iterable of such dictionaries.
+                to add. When provided, inputs/outputs/metadata are extracted automatically.
             dataframe: pandas DataFrame (requires pandas to be installed).
             csv_file_path: Location of a CSV text file
             input_keys: List of column names used as input keys.
             output_keys: List of column names used as output keys.
             metadata_keys: List of column names used as metadata keys.
+            split_keys: List of column names used for automatically assigning examples to splits.
             inputs: List of dictionaries each corresponding to an example.
             outputs: List of dictionaries each corresponding to an example.
             metadata: List of dictionaries each corresponding to an example.
@@ -1412,16 +1523,23 @@ class AsyncDatasets:
         if dataframe is not None and csv_file_path is not None:
             raise ValueError("Please provide either dataframe or csv_file_path, but not both")
 
+        splits_from_examples: list[Any] = []
         if examples is not None:
-            examples_list: list[DatasetExample]
-            if _is_valid_dataset_example(examples):
-                examples_list = [examples]  # type: ignore[list-item]
+            examples_list: list[_InputDatasetExample]
+            if _is_input_dataset_example(examples):
+                examples_list = [examples]
+            elif _is_iterable_of_input_dataset_examples(examples):
+                examples_list = list(examples)
             else:
-                examples_list = list(examples)  # type: ignore[arg-type]
+                raise ValueError(
+                    "examples must be a single dictionary with required 'input' and 'output' keys "
+                    "and an optional 'metadata' key, or an iterable of such dictionaries"
+                )
 
             inputs = [dict(example["input"]) for example in examples_list]
             outputs = [dict(example["output"]) for example in examples_list]
-            metadata = [dict(example["metadata"]) for example in examples_list]
+            metadata = [dict(example.get("metadata", {})) for example in examples_list]
+            splits_from_examples = [example.get("splits", None) for example in examples_list]
 
         if has_tabular:
             table = dataframe if dataframe is not None else csv_file_path
@@ -1432,6 +1550,7 @@ class AsyncDatasets:
                 input_keys=input_keys,
                 output_keys=output_keys,
                 metadata_keys=metadata_keys,
+                split_keys=split_keys,
                 dataset_description=dataset_description,
                 action="create",
                 timeout=timeout,
@@ -1442,6 +1561,7 @@ class AsyncDatasets:
                 inputs=inputs,
                 outputs=outputs,
                 metadata=metadata,
+                splits=splits_from_examples if examples is not None else [],
                 dataset_description=dataset_description,
                 action="create",
                 timeout=timeout,
@@ -1451,12 +1571,13 @@ class AsyncDatasets:
         self,
         *,
         dataset: DatasetIdentifier,
-        examples: Optional[Union[DatasetExample, Iterable[DatasetExample]]] = None,
+        examples: Optional[Union[Mapping[str, Any], Iterable[Mapping[str, Any]]]] = None,
         dataframe: Optional["pd.DataFrame"] = None,
         csv_file_path: Optional[Union[str, Path]] = None,
         input_keys: Iterable[str] = (),
         output_keys: Iterable[str] = (),
         metadata_keys: Iterable[str] = (),
+        split_keys: Iterable[str] = (),
         inputs: Iterable[Mapping[str, Any]] = (),
         outputs: Iterable[Mapping[str, Any]] = (),
         metadata: Iterable[Mapping[str, Any]] = (),
@@ -1468,13 +1589,15 @@ class AsyncDatasets:
         Args:
             dataset: A dataset identifier - can be a dataset ID string, name string,
                 Dataset object, or dict with 'id'/'name' fields.
-            examples: Either a single DatasetExample or list of DatasetExample objects to add.
+            examples: Either a single dictionary with required 'input' and 'output' keys
+                and an optional 'metadata' key, or an iterable of such dictionaries.
                 When provided, inputs/outputs/metadata are extracted automatically.
             dataframe: pandas DataFrame (requires pandas to be installed).
             csv_file_path: Location of a CSV text file
             input_keys: List of column names used as input keys.
             output_keys: List of column names used as output keys.
             metadata_keys: List of column names used as metadata keys.
+            split_keys: List of column names used for automatically assigning examples to splits.
             inputs: List of dictionaries each corresponding to an example.
             outputs: List of dictionaries each corresponding to an example.
             metadata: List of dictionaries each corresponding to an example.
@@ -1518,16 +1641,23 @@ class AsyncDatasets:
         if dataframe is not None and csv_file_path is not None:
             raise ValueError("Please provide either dataframe or csv_file_path, but not both")
 
+        splits_from_examples: list[Any] = []
         if examples is not None:
-            examples_list: list[DatasetExample]
-            if _is_valid_dataset_example(examples):
-                examples_list = [examples]  # type: ignore[list-item]
+            examples_list: list[_InputDatasetExample]
+            if _is_input_dataset_example(examples):
+                examples_list = [examples]
+            elif _is_iterable_of_input_dataset_examples(examples):
+                examples_list = list(examples)
             else:
-                examples_list = list(examples)  # type: ignore[arg-type]
+                raise ValueError(
+                    "examples must be a single dictionary with required 'input' and 'output' keys "
+                    "and an optional 'metadata' key, or an iterable of such dictionaries"
+                )
 
             inputs = [dict(example["input"]) for example in examples_list]
             outputs = [dict(example["output"]) for example in examples_list]
-            metadata = [dict(example["metadata"]) for example in examples_list]
+            metadata = [dict(example.get("metadata", {})) for example in examples_list]
+            splits_from_examples = [example.get("splits") for example in examples_list]
 
         if has_tabular:
             table = dataframe if dataframe is not None else csv_file_path
@@ -1538,6 +1668,7 @@ class AsyncDatasets:
                 input_keys=input_keys,
                 output_keys=output_keys,
                 metadata_keys=metadata_keys,
+                split_keys=split_keys,
                 dataset_description=None,
                 action="append",
                 timeout=timeout,
@@ -1548,6 +1679,7 @@ class AsyncDatasets:
                 inputs=inputs,
                 outputs=outputs,
                 metadata=metadata,
+                splits=splits_from_examples if examples is not None else [],
                 dataset_description=None,
                 action="append",
                 timeout=timeout,
@@ -1584,6 +1716,7 @@ class AsyncDatasets:
         input_keys: Iterable[str],
         output_keys: Iterable[str] = (),
         metadata_keys: Iterable[str] = (),
+        split_keys: Iterable[str] = (),
         dataset_description: Optional[str] = None,
         action: Literal["create", "append"] = "create",
         timeout: Optional[int] = DEFAULT_TIMEOUT_IN_SECONDS,
@@ -1592,6 +1725,7 @@ class AsyncDatasets:
         input_keys_set = frozenset(input_keys)
         output_keys_set = frozenset(output_keys)
         metadata_keys_set = frozenset(metadata_keys)
+        split_keys_set = frozenset(split_keys)
 
         # Auto-infer keys if none provided
         if not any([input_keys_set, output_keys_set, metadata_keys_set]):
@@ -1600,7 +1734,7 @@ class AsyncDatasets:
             output_keys_set = frozenset(output_keys_tuple)
             metadata_keys_set = frozenset(metadata_keys_tuple)
 
-        keys = DatasetKeys(input_keys_set, output_keys_set, metadata_keys_set)
+        keys = DatasetKeys(input_keys_set, output_keys_set, metadata_keys_set, split_keys_set)
 
         if isinstance(table, Path) or isinstance(table, str):
             file = _prepare_csv(Path(table), keys)
@@ -1627,6 +1761,7 @@ class AsyncDatasets:
                 "input_keys[]": sorted(keys.input),
                 "output_keys[]": sorted(keys.output),
                 "metadata_keys[]": sorted(keys.metadata),
+                "split_keys[]": sorted(keys.split),
             },
             params={"sync": True},
             headers={"accept": "application/json"},
@@ -1642,6 +1777,7 @@ class AsyncDatasets:
         inputs: Iterable[Mapping[str, Any]],
         outputs: Iterable[Mapping[str, Any]] = (),
         metadata: Iterable[Mapping[str, Any]] = (),
+        splits: Iterable[Any] = (),
         dataset_description: Optional[str] = None,
         action: Literal["create", "append"] = "create",
         timeout: Optional[int] = DEFAULT_TIMEOUT_IN_SECONDS,
@@ -1651,6 +1787,7 @@ class AsyncDatasets:
         inputs_list = list(inputs)
         outputs_list = list(outputs) if outputs else []
         metadata_list = list(metadata) if metadata else []
+        splits_list = list(splits) if splits else []
 
         if not inputs_list:
             raise ValueError("inputs must be non-empty")
@@ -1658,7 +1795,10 @@ class AsyncDatasets:
         if not _is_all_dict(inputs_list):
             raise ValueError("inputs must contain only dictionaries")
 
-        for name, data in [("outputs", outputs_list), ("metadata", metadata_list)]:
+        for name, data in [
+            ("outputs", outputs_list),
+            ("metadata", metadata_list),
+        ]:
             if data:
                 if len(data) != len(inputs_list):
                     raise ValueError(
@@ -1668,12 +1808,19 @@ class AsyncDatasets:
                 if not _is_all_dict(data):
                     raise ValueError(f"{name} must contain only dictionaries")
 
+        # Validate splits separately (can be string, list of strings, or None)
+        if splits_list and len(splits_list) != len(inputs_list):
+            raise ValueError(
+                f"splits must have same length as inputs ({len(splits_list)} != {len(inputs_list)})"
+            )
+
         payload = {
             "action": action,
             "name": dataset_name,
             "inputs": inputs_list,
             "outputs": outputs_list or [{}] * len(inputs_list),
             "metadata": metadata_list or [{}] * len(inputs_list),
+            "splits": splits_list or [None] * len(inputs_list),
         }
 
         if dataset_description is not None:
@@ -1779,11 +1926,13 @@ def _prepare_dataframe_as_csv(
 
     keys.check_differences(frozenset(df.columns))
 
-    # Ensure consistent column ordering: input, output, metadata
-    selected_columns = sorted(keys.input) + sorted(keys.output) + sorted(keys.metadata)
+    # Ensure consistent column ordering: input, output, metadata, split
+    selected_columns = (
+        sorted(keys.input) + sorted(keys.output) + sorted(keys.metadata) + sorted(keys.split)
+    )
 
     csv_buffer = BytesIO()
-    df[selected_columns].to_csv(csv_buffer, index=False)
+    df[selected_columns].to_csv(csv_buffer, index=False)  # pyright: ignore[reportUnknownMemberType]
     csv_buffer.seek(0)
 
     compressed = BytesIO()
